#!/bin/bash 

#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# 
#  GLite Data Transfer Scripts
#
#  Authors: Paolo Badino <paolo.badino@cern.ch>
#  Version info: $Id: report-common.sh,v 1.8 2007/04/19 11:06:05 badino Exp $
#  Release: $Name:  $
#
#

# Check that GLITE_LOCATION is set
if [ "x$GLITE_LOCATION" == "x" ] ; then
    echo "GLITE_LOCATION Environemnt variable is not set. Please fix your profile";
    exit -1;
fi

# Check that GLITE_LOCATION_VAR is set
if [ "x$GLITE_LOCATION_VAR" == "x" ] ; then
    echo "GLITE_LOCATION_VAR Environment variable is not set. Please fix your profile";
    exit -1;
fi

# Set the default value for the Environemnt File
if [ "x$REPORTS_ENV" == "x" ] ; then
    REPORTS_ENV=$GLITE_LOCATION_VAR/glite-transfer-fts/reports/etc/report-env.sh
fi

# Check that environment script is available
if [ -f "$REPORTS_ENV" ] ; then
    # Source the environment 
    source $REPORTS_ENV
else
    echo "No environment file found in $REPORTS_ENV";
    exit -1;
fi

###############################################################################
# Log finctions
###############################################################################

#
#  log_debug
#
function log_debug {
   if [ "x$DEBUG_ENABLED" != "x" ]; then
       echo "DEBUG: $1"
   fi 
}
#
#  log error and exit
#
function error {
   echo "ERROR: $1"
   exit 1
}

###############################################################################
# Set the Default Values
###############################################################################

# Python
if [ "x$PYTHON" == "x" ] ; then
    PYTHON=/usr/bin/python
fi

# xsltproc
if [ "x$XSLTPROC" == "x" ] ; then
    XSLTPROC=/usr/bin/xsltproc
fi

# Set the FTS isntance name
if [ "x$REPORTS_FTS_INSTANCE" == "x" ] ; then
    REPORTS_FTS_INSTANCE=`hostname`
fi

# Set the directory where the XML reports will be stored
if [ "x$REPORTS_OUT_DIR" == "x" ] ; then
    REPORTS_OUT_DIR=$GLITE_LOCATION_VAR/glite-transfer-fts/reports/xml
fi

# Set the directory where the HTML reports will be stored
if [ "x$REPORTS_HTML_DIR" == "x" ] ; then
    REPORTS_HTML_DIR=$GLITE_LOCATION_VAR/glite-transfer-fts/reports/html
fi

# Set the relative path from the webroot where the images, css and js files are stored
if [ "x$REPORTS_HTML_WEBROOT_PATH" == "x" ] ; then
    REPORTS_HTML_WEBROOT_PATH=""
fi

# Set the directory where the HTML reports will be stored
if [ "x$REPORTS_HTML_WEBROOT_PATH" == "x" ] ; then
    REPORTS_HTML_WEBROOT_PATH=''
fi

# FTS Report Scripts Directory
if [ "x$REPORTS_SCRIPTS_DIR" == "x" ] ; then
    REPORTS_SCRIPTS_DIR=$GLITE_LOCATION/share/glite-data-transfer-scripts/reports
fi

if [ "x$REPORT_REL_DIR" == "x" ] ; then
    REPORT_REL_DIR=`date +'%Y/%m/%d'`
fi

# Include Oracle lib in LD_LIBRARY_PATH
if [ "x$ORACLE_LOCATION" != "x" ] ; then
    # Check that the folder exists
    if [ -d $ORACLE_LOCATION/client/lib ] ; then
        export LD_LIBRARY_PATH=$ORACLE_LOCATION/client/lib:$LD_LIBRARY_PATH
    else
        # check with ORACLE_LOCATION/lib
        if [ -d $ORACLE_LOCATION/lib ] ; then
            export LD_LIBRARY_PATH=$ORACLE_LOCATION/lib:$LD_LIBRARY_PATH
        else
            echo "neither $ORACLE_LOCATION/client/lib nor $ORACLE_LOCATION/lib exist";
            exit -1;
        fi
    fi
fi


# Set report Directory    
REPORT_DIR=$REPORTS_OUT_DIR/$REPORT_CATEGORY/$REPORT_TYPE/$REPORT_INTERVAL/$REPORT_REL_DIR

# Print the configuration information
log_debug "GLITE_LOCATION:            $GLITE_LOCATION";
log_debug "GLITE_LOCATION_VAR:        $GLITE_LOCATION_VAR";
log_debug "PYTHON:                    $PYTHON";
log_debug "XSLTPROC:                  $XSLTPROC";
log_debug "REPORTS_FTS_INSTANCE:      $REPORTS_FTS_INSTANCE";
log_debug "REPORTS_OUT_DIR:           $REPORTS_OUT_DIR";
log_debug "REPORTS_HTML_DIR:          $REPORTS_HTML_DIR";
log_debug "REPORTS_HTML_WEBROOT_PATH: $REPORTS_HTML_WEBROOT_PATH";
log_debug "REPORTS_SCRIPTS_DIR:       $REPORTS_SCRIPTS_DIR";
log_debug "REPORT_CATEGORY:           $REPORT_CATEGORY";
log_debug "REPORT_TYPE:               $REPORT_TYPE";
log_debug "REPORT_INTERVAL:           $REPORT_INTERVAL";
log_debug "REPORT_REL_DIR:            $REPORT_REL_DIR";
log_debug "REPORT_DIR:                $REPORT_DIR";
log_debug "PYTHONPATH:                $PYTHONPATH";
log_debug "LD_LIBRARY_PATH:           $LD_LIBRARY_PATH";

###############################################################################
# Functions
###############################################################################

#
# check preconditions
#
function check_preconditions {
    if [ "x$REPORTS_FTS_INSTANCE" == "x" ]; then
        error "No FTS Instance specified"
    fi
    if [ "x$REPORTS_OUT_DIR" == "x" ]; then
        error "No Reports Output Directory set"
    fi
    if [ "x$REPORTS_HTML_DIR" == "x" ]; then
        error "No Reports HTML Output Directory set"
    fi
    if [ "x$REPORT_CATEGORY" == "x" ]; then
        error "No Report Category set"
    fi
    if [ "x$REPORT_TYPE" == "x" ]; then
        error "No Report Type set"
    fi
    if [ "x$REPORT_INTERVAL" == "x" ]; then
        error "No Report Interval set"
    fi
    if [ "x$REPORT_REL_DIR" == "x" ]; then
        error "No Report Relative Path set"
    fi
    if [ "x$REPORT_DIR" == "x" ]; then
        error "No Report Path set"
    fi
}

#
# Report Validation
#
function test_report {

    # Check Preconditions
    check_preconditions

    log_debug "Run Report Validation"

    # Run Report 
    log_debug "$PYTHON $REPORTS_SCRIPTS_DIR/fts-report.py -u $DB_USER -p $DB_PASSWORD -c \"$DB_CONNECT_STR\" -Z"
    $PYTHON $REPORTS_SCRIPTS_DIR/fts-report.py -u $DB_USER -p $DB_PASSWORD -c "$DB_CONNECT_STR" -Z
    if [ $? -ne 0 ]; then
        error "Failed to run report validation"
    fi

    log_debug "Report Validation Success"
}

#
# make Report
#
function make_report {

    # Check Preconditions
    check_preconditions

    # Set Report Output Directory
    if [ ! -d $REPORT_DIR ]; then
        mkdir -p $REPORT_DIR
    fi
    
    log_debug "Generating report $REPORT_CATEGORY:$REPORT_TYPE:$REPORT_INTERVAL"

    # Run Report 
    log_debug "$PYTHON $REPORTS_SCRIPTS_DIR/fts-report.py -u $DB_USER -p $DB_PASSWORD -c \"$DB_CONNECT_STR\" -o \"$REPORT_DIR\" -t $REPORT_TYPE -i $REPORT_INTERVAL -I \"$REPORTS_FTS_INSTANCE\""
    $PYTHON $REPORTS_SCRIPTS_DIR/fts-report.py -u $DB_USER -p $DB_PASSWORD -c "$DB_CONNECT_STR" -o "$REPORT_DIR" -t $REPORT_TYPE -i $REPORT_INTERVAL -I "$REPORTS_FTS_INSTANCE"
    if [ $? -ne 0 ]; then
        error "Failed to generate report $REPORT_CATEGORY:$REPORT_TYPE:$REPORT_INTERVAL"
    fi

    log_debug "Report Generated"
}

#
# Make html
#
function make_html {

    # Check Preconditions
    check_preconditions

    # Create Output HTML Directory
    HTML_ROOT=$REPORTS_HTML_DIR/$REPORT_CATEGORY/$REPORT_TYPE/$REPORT_INTERVAL
    HTML_REL_DIR=$REPORT_REL_DIR
    HTML_DIR=$HTML_ROOT/$HTML_REL_DIR
    if [ ! -d $HTML_DIR ]; then
        mkdir -p $HTML_DIR
        if [ $? -ne 0 ]; then
            error "Cannot create dir $HTML_DIR"
        fi
    fi
    HTML_REL_PATH=`perl -e "@values = split('/','$HTML_REL_DIR'); foreach $v (@values) { print '../';}"`
    log_debug "HTML Report dir is $HTML_DIR"
    log_debug "HTML Relative path is $HTML_REL_DIR"
    log_debug "HTML relative back path is $HTML_REL_PATH"
 
    # Copy the report html index and CSS file in the destination directory
    LINK_NAME=index.html
    CSS_NAME=report.css
    JS_NAME=report.js
    IMAGES_NAME=images
    # Generate report index
    TMP_INDEX_FILE=`mktemp`
    PATH_ESCAPED=`echo ${REPORTS_HTML_WEBROOT_PATH} | sed 's/\//\\\\\//g'`
    INSTANCE_ESCAPED=`echo ${REPORTS_FTS_INSTANCE}  | sed 's/\//\\\\\//g'`
    log_debug "sed \"s/@PATH_FROM_WEBROOT@/${PATH_ESCAPED}/g; s/@FTS_INSTANCE_NAME@/${INSTANCE_ESCAPED}/g\" $REPORTS_SCRIPTS_DIR/html/$LINK_NAME > $TMP_INDEX_FILE"
    sed "s/@PATH_FROM_WEBROOT@/${PATH_ESCAPED}/g; s/@FTS_INSTANCE_NAME@/${INSTANCE_ESCAPED}/g" $REPORTS_SCRIPTS_DIR/html/$LINK_NAME > $TMP_INDEX_FILE
    # Move the file
    mv ${TMP_INDEX_FILE} ${REPORTS_HTML_DIR}/${LINK_NAME}
    chmod 0644 ${REPORTS_HTML_DIR}/${LINK_NAME}
    log_debug "File ${REPORTS_HTML_DIR}/${LINK_NAME} generated"

    # overwrite the other files
    cp $REPORTS_SCRIPTS_DIR/html/$CSS_NAME $REPORTS_HTML_DIR/$CSS_NAME
    cp $REPORTS_SCRIPTS_DIR/html/$JS_NAME $REPORTS_HTML_DIR/$JS_NAME
    if [ ! -d $REPORTS_HTML_DIR/$IMAGES_NAME ] ; then
        mkdir $REPORTS_HTML_DIR/$IMAGES_NAME
    fi
    cp $REPORTS_SCRIPTS_DIR/html/$IMAGES_NAME/* $REPORTS_HTML_DIR/$IMAGES_NAME
    
    # Get the last touched report
    REPORTS=`find $REPORT_DIR -type f | egrep -e '.*\.xml$'`
    if [ "x$REPORTS" == "x" ]; then
       error "No report available: Can't generate HTML page"
    fi
    REPORTS=`ls -1 -t $REPORTS`
    for rep in $REPORTS ; do 
       REPORT_NAME=$rep
       break
    done
    log_debug "Report is $REPORT_NAME"

    # Generate the name for the HTML
    HTML_FILE_NAME=`basename $REPORT_NAME | sed 's/\.xml/\.html/'`
    log_debug "HTML DIR is $HTML_DIR"
    log_debug "HTML file name is $HTML_FILE_NAME"
    HTML_NAME=$HTML_DIR/$HTML_FILE_NAME
    HTML_REL_NAME=$HTML_REL_DIR/$HTML_FILE_NAME
    log_debug "HTML relative file name is $HTML_REL_NAME"
    HTML_NAME=$HTML_DIR/$HTML_FILE_NAME

    # Convert the report in html
    WEBROOT_PATH_OPTION=''
    if [ "x$REPORTS_HTML_WEBROOT_PATH" != "x" ]; then
        WEBROOT_PATH_OPTION="--stringparam WebRootPath ${REPORTS_HTML_WEBROOT_PATH}"
    fi
    log_debug "$XSLTPROC $WEBROOT_PATH_OPTION $REPORTS_SCRIPTS_DIR/report-to-html.xsl $REPORT_NAME > $HTML_NAME"
    $XSLTPROC $WEBROOT_PATH_OPTION $REPORTS_SCRIPTS_DIR/report-to-html.xsl $REPORT_NAME > $HTML_NAME
    if [ $? -ne 0 ]; then
        error "Failed to produce HTML report $HTML_NAME"
    fi

    # Move the symlink to the last one
    ln -f -s $HTML_REL_NAME $HTML_ROOT/$LINK_NAME
}

